@@ -4,13 +4,14 @@
 # Remove dateutil/req from pagegetter
 # Make aws_sdb_reservation_domain optional
 # Make aws cache bucket optional
+# Delete myself function call
 
 from twisted.python.failure import Failure
 
 from uuid import uuid4, uuid5, NAMESPACE_DNS
 from twisted.internet import task
 from twisted.internet import reactor
-from twisted.internet.defer import Deferred, DeferredList
+from twisted.internet.defer import Deferred, DeferredList, maybeDeferred
 
 from pagegetter import RequestQueuer, PageGetter
 from unicodeconverter import convertToUTF8
@@ -215,12 +216,10 @@ class AWSpider:
             "cost":cost,
             "active_requests_by_host":active_requests_by_host,
             "pending_requests_by_host":pending_requests_by_host,
-            "active_requests":self.active,
-            "pending_requests":self.pending
+            "active_requests":self.rq.active,
+            "pending_requests":self.rq.pending
         }
         
-        
-        
         logger.debug("Got server data:\n%s" % PrettyPrinter.pformat(data) )
 
         return data
@@ -329,10 +328,8 @@ class AWSpider:
             if "call_immediately" in kwargs and not evaluateBoolean( kwargs["call_immediately"] ):    
                 d = DeferredList([a], consumeErrors=True)
             else:
-                logger.debug( "Calling %s in a thread immediately with arguments:\n%s" % (function_name, PrettyPrinter.pformat(filtered_kwargs) ) )
-                b = Deferred()
-                b.addCallback(self._callExposedFunctionWrapper, function["function"], filtered_kwargs, function_name, uuid)
-                reactor.callInThread( b.callback, None )
+                logger.debug( "Calling %s immediately with arguments:\n%s" % (function_name, PrettyPrinter.pformat(filtered_kwargs) ) )
+                b = self.callExposedFunctionImmediately( function["function"], filtered_kwargs, function_name )
                 d = DeferredList([a,b], consumeErrors=True)
             
             d.addCallback( self._createReservationCallback2, function_name, uuid )
@@ -340,12 +337,11 @@ class AWSpider:
             return d
             
         else:
-            logger.debug( "Calling %s in a thread immediately with arguments:\n%s" % (function_name, PrettyPrinter.pformat(filtered_kwargs) ) )
-            d = Deferred()
-            d.addCallback(self.callExposedFunctionImmediately, function["function"], filtered_kwargs, function_name)
-            reactor.callInThread( d.callback, None )
+            logger.debug( "Calling %s immediately with arguments:\n%s" % (function_name, PrettyPrinter.pformat(filtered_kwargs) ) )
+            d = self.callExposedFunctionImmediately( function["function"], filtered_kwargs, function_name )
             return d
 
+
     def _callExposedFunctionWrapper( self, data, func, kwargs, function_name, uuid ):
         return self.callExposedFunction( func, kwargs, function_name, uuid )
 
@@ -372,19 +368,12 @@ class AWSpider:
         logger.error( "Unable to create reservation for %s:%s, %s.\n" % (function_name, uuid, error) )
         return error
     
-    def callExposedFunctionImmediately( self, data, func, kwargs, function_name ):
-
-        try:
-            d = func( **kwargs )
-        except Exception, e:
-            return self._callExposedFunctionImmediatelyErrback( Failure(exc_value=sys.exc_value, exc_type=sys.exc_type, exc_tb=sys.exc_traceback), function_name )
-            
-        if isinstance( d, Deferred ):            
-            d.addCallback( self._callExposedFunctionImmediatelyCallback, function_name )
-            d.addErrback( self._callExposedFunctionImmediatelyErrback, function_name )
-            return d
-        else:
-            return self._callExposedFunctionImmediatelyCallback( d, function_name )
+    def callExposedFunctionImmediately( self, func, kwargs, function_name ):
+        
+        d = maybeDeferred( func, **kwargs )
+        d.addCallback( self._callExposedFunctionImmediatelyCallback, function_name )
+        d.addErrback( self._callExposedFunctionImmediatelyErrback, function_name )
+        return d
         
     def _callExposedFunctionImmediatelyCallback( self, data, function_name ):
         logger.debug("Function %s returned successfully." % (function_name) )
